@@ -4,10 +4,8 @@ from datetime import datetime
 from pathlib import Path
 
 import click
+from threedi_api_client.openapi.api.v3_api import V3Api
 import websockets
-from openapi_client.api.simulations_api import SimulationsApi
-from openapi_client.api.organisations_api import OrganisationsApi
-from openapi_client.api.threedimodels_api import ThreedimodelsApi
 from openapi_client import Configuration, ApiException
 from rich.table import Table
 from rich import box
@@ -43,7 +41,7 @@ app.add_typer(api_app, name="api")
 @api_app.command()
 def models(ctx: typer.Context):
     """List available threedimodels"""
-    threedi_models_api = ThreedimodelsApi(ctx.obj.api_client)
+    threedi_models_api: V3Api = ctx.obj.api_client
     threedi_models = threedi_models_api.threedimodels_list()
 
     table = Table(
@@ -89,7 +87,7 @@ def models(ctx: typer.Context):
 def organisations(
         ctx: typer.Context):
     """List available organisations"""
-    organisations_api = OrganisationsApi(ctx.obj.api_client)
+    organisations_api: V3Api = ctx.obj.api_client
     organisations = organisations_api.organisations_list()
     table = Table(
         show_header=True,
@@ -132,7 +130,7 @@ def simulations(
         )
 ):
     """List simulations"""
-    simulations_api = SimulationsApi(ctx.obj.api_client)
+    simulations_api: V3Api = ctx.obj.api_client
     simulations = simulations_api.simulations_list()
 
     table = Table(
@@ -257,32 +255,41 @@ def run_scenario(
         scenario = IntPrompt.ask(
             "Which scenario do you want to run? ID"
         )
-    if model is None:
-        models(ctx)
-        model = IntPrompt.ask("Which model do you want to run? ID")
+
+    scenario_to_run = ctx.obj.scenarios[scenario]["file"]
+    parser = ScenarioParser(scenario_to_run)
+
+    if parser.is_simulation_scenario:
+        if model is None:
+            models(ctx)
+            model = IntPrompt.ask("Which model do you want to run? ID")
+
+    
     if not organisation:
         organisations(ctx)
         organisation = Prompt.ask("Which organisation do you want to use? UUID")
-    scenario_to_run = ctx.obj.scenarios[scenario]["file"]
+
     name = ctx.obj.scenarios[scenario]["name"]
+
     context = {
         "threedimodel_id": model,
         "organisation_uuid": organisation,
         "simulation_name": name,
+        "schematisation_name": name,
         "datetime_now": datetime.utcnow().isoformat(),
     }
-    parser = ScenarioParser(scenario_to_run, context)
+
     try:
         scenario = parser.parse(
-            ctx.obj.api_client, ctx.obj.websocket_settings
+            ctx.obj.api_client, ctx.obj.websocket_settings, context=context
         )
     except (ResolveError, ApiModelError, LoadScenarioError) as err:
         console.print(f":collision: {err}", style="error")
         sys.exit(ExitCodes.SCENARIO_CONFIG_ERROR.value)
 
     console.rule(f"Loading scenario {name}", style="bold blue")
-    scenario.simulation.save()
-    console.print(f":link: URL: {scenario.simulation.instance.url}")
+    scenario.base_object.save()
+    console.print(f":link: URL: {scenario.base_object.instance.url}")
 
     try:
         console.rule(f"Starting scenario run...", style="bold blue")
@@ -347,9 +354,8 @@ def results(
         )
 
     try:
-        simulations_api = SimulationsApi(ctx.obj.api_client)
-        threedimodels_api = ThreedimodelsApi(ctx.obj.api_client)
-
+        simulations_api: V3Api = ctx.obj.api_client
+        threedimodels_api = simulations_api
         simulation = simulations_api.simulations_read(id=simulation)
     except ApiException as err:
         console.print(f"{err}", style="error")
