@@ -14,14 +14,21 @@ from rich.prompt import Prompt
 
 from openapi_client import Configuration, ApiClient, ApiException
 from threedi_api_client.threedi_api_client import ThreediApiClient
+from threedi_api_client.api import ThreediApi
 from threedi_api_client.aio.threedi_api_client import (
     ThreediApiClient as AsyncThreediApiClient,
 )
 
-from threedi_api_client.threedi_api_client import (
-    get_auth_token,
-    refresh_api_key,
-)
+try:
+    from threedi_api_client.auth import (
+        get_auth_token,
+        refresh_api_key,
+    )
+except ImportError:
+    from threedi_api_client.threedi_api_client import (
+        get_auth_token,
+        refresh_api_key,
+    )
 
 from threedi_cmd.console import console
 from threedi_cmd.errors import ExitCodes
@@ -30,9 +37,9 @@ SCENARIO_DIR = Path(__file__).parent.parent / "scenarios"
 
 
 class EndpointOption(Enum):
-    localhost = "http://localhost:8000/v3.0"
-    staging = "https://api.staging.3di.live/v3.0"
-    production = "https://api.3di.live/v3.0"
+    localhost = "http://localhost:8000"
+    staging = "https://api.staging.3di.live"
+    production = "https://api.3di.live"
 
 
 @lru_cache()
@@ -124,15 +131,16 @@ class Settings:
     APP_NAME = "3di-scenario"
     CONFIG_FILE_NAME = "3di_config.yaml"
 
-    def __init__(self, endpoint: str = ""):
+    def __init__(self, endpoint: str = "", websocket_api_version='v3-beta'):
         self._endpoint = EndpointOption[endpoint]
         self.cached_config = CachedConfig.load_from_file(self.config_file)
         try:
             refresh_api_key(self.configuration)
         except ValueError:
             self.credentials_prompt(self.configuration)
+        
         self.websocket_settings = WebSocketSettings(
-            api_base_url=self.endpoint, token=self.access
+            api_base_url=self.endpoint + '/' + websocket_api_version, token=self.access
         )
 
     @property
@@ -220,7 +228,15 @@ class Settings:
 
     @property
     def api_client(self) -> ThreediApiClient:
-        client = ApiClient(self.configuration)
+        client = ThreediApi(
+            config={
+                "THREEDI_API_HOST": self.endpoint,
+                "THREEDI_API_USERNAME": self.username,
+                "THREEDI_API_ACCESS_TOKEN": self.access,
+                "THREEDI_API_REFRESH_TOKEN": self.refresh
+            }
+            , version='v3-beta'
+        )
         return client
 
     @property
