@@ -5,6 +5,8 @@ from typing import Dict
 from openapi_client import ApiClient
 from openapi_client.models import Simulation
 from openapi_client.api import SimulationsApi
+from threedi_api_client.openapi.api import V3BetaApi
+from threedi_api_client.openapi.models import Schematisation
 from pathlib import Path
 import requests
 from rich.live import Live
@@ -34,11 +36,13 @@ def class_repr(data, model):
 
 class BaseWrapper:
     def __init__(
-        self, data: Dict, api_client: ApiClient, base_path: Path = None
+        self, data: Dict, api_client: ApiClient, base_path: Path = None,
+        refs: Dict = None,
     ):
         self._api_client = api_client
         self._data = data
         self._base_path = base_path
+        self._refs = refs
 
     async def execute(self, queue: Queue):
         pass
@@ -48,7 +52,7 @@ class BaseWrapper:
         return []
 
 
-BLACKLIST = {"waitfor_timeout"}
+BLACKLIST = {"waitfor_timeout", }
 
 
 class ModelWrapper(BaseWrapper):
@@ -60,18 +64,27 @@ class ModelWrapper(BaseWrapper):
     scenario_name = None
 
     def __init__(
-        self, data: Dict, api_client: ApiClient, base_path: Path = None
+        self, *args, **kwargs
     ):
-        super().__init__(data, api_client, base_path)
-        self.initialize_instance(data)
-        self.api = self.api_class(api_client)  # noqa
+        super().__init__(*args, **kwargs)
+        self.initialize_instance(self._data)
+        self.api = self._api_client  # noqa
+
+        # Add self to refs
+        if '_ref' in self._data:
+            self._refs[self._data['_ref']] = self.instance
+
+    def resolve_func_name(self, suffix: str):
+        func = getattr(self._api_client, f"{self.api_path}{suffix}")
+        return func
+
 
     def initialize_instance(self, data: Dict):
         """raises ApiModelError if initializing the instance fails"""
         new_data = {
             key: value
             for key, value in data.items()
-            if not key.startswith("__") and key not in BLACKLIST
+            if not key.startswith("_") and key not in BLACKLIST
         }
         try:
             self.instance = self.model(**new_data)
@@ -88,7 +101,7 @@ class ModelWrapper(BaseWrapper):
     def create(self):
         with Live(console=console) as live:
             live.update(f"Creating {self.model.__name__}...")
-            func = getattr(self.api, f"{self.api_path}_create")
+            func = self.resolve_func_name("_create")
             res = func(self.instance)
             live.update(
                 f":heavy_check_mark: [bold spring_green4] Created {self.model.__name__}"
@@ -99,7 +112,7 @@ class ModelWrapper(BaseWrapper):
     def patch(self, data):
         with Live(console=console) as live:
             live.update(f"Patching {self.model.__name__}...")
-            func = getattr(self.api, f"{self.api_path}_partial_update")
+            func = self.resolve_func_name("_partial_update")
             res = func(self.instance.id, data)
             live.update(
                 f":heavy_check_mark: [bold spring_green4] Patched {self.model.__name__}"
@@ -109,8 +122,8 @@ class ModelWrapper(BaseWrapper):
 
     def delete(self):
         with Live(console=console) as live:
-            live.update(f"Patching {self.model.__name__}...")
-            func = getattr(self.api, f"{self.api_path}_delete")
+            live.update(f"Deleting {self.model.__name__}...")
+            func = self.resolve_func_name("_delete")
             res = func(self.instance.id)
             live.update(
                 f":heavy_check_mark: [bold spring_green4] Deleted {self.model.__name__}"
@@ -153,28 +166,34 @@ class SimulationWrapper(ModelWrapper):
     api_path = "simulations"
 
 
+class SchematisationWrapper(ModelWrapper):
+    api_class = V3BetaApi
+    model = Schematisation
+    api_path = "schematisations"
+
+
 class SimulationChildWrapper(ModelWrapper):
     simulation = None
     base_path = "simulations_"
 
     def __init__(
-        self,
-        data: Dict,
-        api_client: ApiClient,
-        simulation: Simulation,
-        base_path: Path = None,
+        self, * args, **kwargs
     ):
-        self.simulation = simulation
-        super().__init__(data, api_client, base_path)
+        self.simulation = kwargs.pop('simulation', None)
+        super().__init__(*args, **kwargs)
 
     @property
     def instance_name(self):
         return self.scenario_name
 
+    def resolve_func_name(self, suffix: str):
+        func = getattr(self._api_client, f"{self.base_path}{self.api_path}{suffix}")
+        return func
+
     def create(self, *args, **kwargs):
         with Live(console=console) as live:
             live.update(f"Creating {self.model.__name__}...")
-            func = getattr(self.api, f"{self.base_path}{self.api_path}_create")
+            func = self.resolve_func_name("_create")
             res = func(self.simulation.id, self.instance, *args, **kwargs)
             live.update(
                 f":heavy_check_mark: [bold spring_green4] Created {self.model.__name__}"
@@ -185,9 +204,7 @@ class SimulationChildWrapper(ModelWrapper):
     def patch(self, data):
         with Live(console=console) as live:
             live.update(f"Patching {self.model}...")
-            func = getattr(
-                self.api, f"{self.base_path}{self.api_path}_partial_update"
-            )
+            func = self.resolve_func_name("_partial_update")
             res = func(self.instance.id, self.simulation.id, data)
             live.update(
                 f":heavy_check_mark: [bold spring_green4] Patched {self.model}"
@@ -195,7 +212,7 @@ class SimulationChildWrapper(ModelWrapper):
         return res
 
     def delete(self):
-        func = getattr(self.api, f"{self.base_path}{self.api_path}_delete")
+        func = self.resolve_func_name("_delete")
         res = func(self.instance.id, self.simulation.id)
         logger.debug(
             "Deleted %s with id: %s", self.instance_name, self.instance.id
@@ -203,17 +220,142 @@ class SimulationChildWrapper(ModelWrapper):
         return res
 
 
+
+class SchematisationChildWrapper(ModelWrapper):
+    schematisation = None
+    base_path = "schematisations_"
+
+    def __init__(
+        self, * args, **kwargs
+    ):
+        self.schematisation = kwargs.pop('schematisation', None)
+        super().__init__(*args, **kwargs)
+
+    def resolve_func_name(self, suffix: str):
+        func = getattr(self._api_client, f"{self.base_path}{self.api_path}{suffix}")
+        return func
+
+    @property
+    def instance_name(self):
+        return self.scenario_name
+
+    def create(self, *args, **kwargs):
+        with Live(console=console) as live:
+            live.update(f"Creating {self.model.__name__}...")
+            func = self.resolve_func_name("_create")
+            res = func(self.schematisation.id, self.instance, *args, **kwargs)
+            live.update(
+                f":heavy_check_mark: [bold spring_green4] Created {self.model.__name__}"
+            )
+        console.print(self._data)
+        return res
+
+    def patch(self, data):
+        with Live(console=console) as live:
+            live.update(f"Patching {self.model}...")
+            func = self.resolve_func_name("_partial_update")
+            res = func(self.instance.id, self.schematisation.id, data)
+            live.update(
+                f":heavy_check_mark: [bold spring_green4] Patched {self.model}"
+            )
+        return res
+
+    def delete(self):
+        func = self.resolve_func_name("_delete")
+        res = func(self.instance.id, self.schematisation.id)
+        logger.debug(
+            "Deleted %s with id: %s", self.instance_name, self.instance.id
+        )
+        return res
+
+
+class RevisionChildWrapper(SchematisationChildWrapper):
+    def __init__(
+        self, * args, **kwargs
+    ): 
+        super().__init__(*args, **kwargs)
+        
+    def resolve_func_name(self, suffix: str):
+        func = getattr(self._api_client, f"{self.base_path}{self.api_path}{suffix}")
+        return func
+
+    @property
+    def revision(self):
+        """
+        Try to resolve revision based
+        on _revision reference
+        """
+        revision_ref = self._data['_revision']
+        return self._refs[revision_ref]
+
+    @property
+    def instance_name(self):
+        return self.scenario_name
+
+    def create(self, *args, **kwargs):
+        with Live(console=console) as live:
+            live.update(f"Creating {self.model.__name__}...")
+            func = self.resolve_func_name("_create")
+            res = func(self.revision.id, self.schematisation.id, self.instance, *args, **kwargs)
+            live.update(
+                f":heavy_check_mark: [bold spring_green4] Created {self.model.__name__}"
+            )
+        console.print(self._data)
+        return res
+
+    def patch(self, data):
+        with Live(console=console) as live:
+            live.update(f"Patching {self.model}...")
+            func = self.resolve_func_name("_partial_update")
+            res = func(self.instance.id, self.schematisation.id, data)
+            live.update(
+                f":heavy_check_mark: [bold spring_green4] Patched {self.model}"
+            )
+        return res
+
+    def delete(self):
+        func = self.resolve_func_name("_delete")
+        res = func(self.instance.id, self.schematisation.id)
+        logger.debug(
+            "Deleted %s with id: %s", self.instance_name, self.instance.id
+        )
+        return res 
+
+    def _upload_file(self, put_url=None):
+        if not hasattr(self, "filepath") or not self.filepath:
+            return
+        if put_url is None:
+            put_url = self.instance.put_url
+
+        filepath = Path(self.filepath)
+        if self._base_path:
+            filepath = self._base_path / filepath
+        fp = (filepath).resolve()
+        # Upload file
+        with fp.open("rb") as f:
+            res = requests.put(put_url, data=f)
+            assert res.status_code == 200
+
+    def save(self, upload_file=True):
+        super().save()
+
+        if upload_file:
+            self._upload_file()
+
+
 class EventWrapper(SimulationChildWrapper):
     base_path = "simulations_events_"
 
     def save(self):
         super().save()
+
         if not hasattr(self, "filepath") or not self.filepath:
             return
         filepath = Path(self.filepath)
         if self._base_path:
             filepath = self._base_path / filepath
         fp = (filepath).resolve()
+
         # Upload file
         with fp.open("rb") as f:
             res = requests.put(self.instance.put_url, data=f)
