@@ -1,14 +1,16 @@
 import asyncio
 from functools import cached_property
 from pathlib import Path
-from typing import Dict, List
+from typing import Dict, List, Any
 
 from openapi_client.models import Simulation
-from openapi_client.api import SimulationsApi
+from threedi_api_client.openapi.models import Schematisation
+from threedi_api_client.openapi.api import V3Api
 from openapi_client.exceptions import ApiException
 from threedi_api_client.threedi_api_client import ThreediApiClient
 
 from .base import SimulationWrapper, ModelWrapper, BaseWrapper
+from .base import SchematisationWrapper
 from threedi_cmd.console import console
 from threedi_cmd.websockets.clients import WebsocketClient
 from threedi_cmd.commands.settings import WebSocketSettings
@@ -23,9 +25,13 @@ class ResolveError(Exception):
     pass
 
 
+
 class Scenario:
-    simulation: Simulation = None
-    steps: any = []
+    base_object: Any = None
+    steps: Any = []   
+
+    # Internal instances administration
+    _refs: Dict = None
 
     def __init__(
         self,
@@ -35,6 +41,7 @@ class Scenario:
         websocket_settings: WebSocketSettings,
         base_path: Path = None,
     ):
+        self._refs = {}
         self.wrappers = wrappers
         self.websocket_settings = websocket_settings
         self.threedi_api_client = threedi_api_client
@@ -42,28 +49,28 @@ class Scenario:
         scenario = data.get("scenario")
         if not scenario:
             raise LoadScenarioError("Could not load scenario")
-        simulation_data = scenario.get("simulation")
         steps_data = scenario.get("steps", [])
-        self.create_simulation(simulation_data)
+        self.create_base_object(scenario)
         self.steps = self.create_steps(steps_data)
 
+    def create_base_object(self, scenario: Dict):
+        pass
+
+    @property
+    def instance_kwargs(self):
+        raise NotImplementedError("Implement in subclass")
+
+    def cleanup(self):
+        raise NotImplementedError("Implement in subclass")
+
+    @property
+    def websocket_endpoint_uri(self):
+        raise NotImplementedError("Implement in subclass")
+
     @cached_property
     def websocket_client(self):
         return WebsocketClient(settings=self.websocket_settings)
 
-    def create_simulation(self, simulation_data: Dict):
-        self.simulation = SimulationWrapper(
-            simulation_data, self.threedi_api_client
-        )
-
-    def shutdown_simulation(self):
-        console.print("• Requesting simulation shutdown")
-        api = SimulationsApi(self.threedi_api_client)
-
-        api.simulations_actions_create(
-            self.simulation.instance.id, {"name": "shutdown"}
-        )
-
     def create_steps(self, steps_data: List) -> List[ModelWrapper]:
         """:raises ResolveError if the scenario name can not be found in `steps_data`"""
         steps = []
@@ -81,8 +88,9 @@ class Scenario:
             wrapped_instance = wrapper_class(
                 data,
                 self.threedi_api_client,
-                simulation=self.simulation.instance,
                 base_path=self._base_path,
+                refs=self._refs,
+                **self.instance_kwargs
             )
             steps.append(wrapped_instance)
         return steps
@@ -106,10 +114,7 @@ class Scenario:
             raise FailedStep(msg)
         finally:
             # Always try to cleanup the simulation
-            try:
-                self.shutdown_simulation()
-            except Exception:
-                pass
+            self.cleanup()
 
         # Done, close the websocket connection
         try:
@@ -118,12 +123,12 @@ class Scenario:
             console.print(e, style="error")
 
     async def execute(self):
-        # Create simulation
-        if self.simulation.instance.id is None:
-            self.simulation.save()
-        endpoint_uri = f"simulations/{self.simulation.instance.id}/?scenario_test_framework=true"
+        # Create base object (Simulation or Schematisation)
+        if self.base_object.instance.id is None:
+            self.base_object.save()
+
         listen_task = asyncio.create_task(
-            self.websocket_client.listen(endpoint_uri)
+            self.websocket_client.listen(self.websocket_endpoint_uri)
         )
         try:
             await asyncio.wait_for(
@@ -138,11 +143,75 @@ class Scenario:
             await asyncio.gather(listen_task)
             return
         run_task = asyncio.create_task(self._run())
+
         finished, _ = await asyncio.wait(
             [listen_task, run_task], return_when=asyncio.FIRST_EXCEPTION
         )
+
         for result in finished:
             if not result:
                 continue
             if result.exception():
                 raise result.exception()
+
+
+
+class SimulationScenario(Scenario):
+    base_object: Simulation = None
+
+    @property
+    def simulation(self):
+        return self.base_object
+
+    def cleanup(self):
+        try:
+            self.shutdown_simulation()
+        except Exception:
+            pass
+
+    def create_base_object(self, scenario: Dict):
+        simulation_data = scenario.get("simulation")
+        self.base_object = SimulationWrapper(
+            simulation_data, self.threedi_api_client
+        )
+
+    @property
+    def websocket_endpoint_uri(self):
+        return f"simulations/{self.simulation.instance.id}/?scenario_test_framework=true"
+
+    @property
+    def instance_kwargs(self):
+        return {'simulation': self.simulation.instance}
+
+    def shutdown_simulation(self):
+        console.print("• Requesting simulation shutdown")
+        api: V3Api = self.threedi_api_client
+
+        api.simulations_actions_create(
+            self.simulation.instance.id, {"name": "shutdown"}
+        )
+
+
+class SchematisationScenario(Scenario):
+    base_object: Schematisation = None
+
+    def cleanup(self):
+        pass
+
+    @property
+    def schematisation(self):
+        return self.base_object
+
+    def create_base_object(self, scenario: Dict):
+        schematisation_data = scenario.get("schematisation")
+        self.base_object = SchematisationWrapper(
+            schematisation_data, self.threedi_api_client
+        )
+
+    @property
+    def websocket_endpoint_uri(self):
+        return f"schematisations/{self.schematisation.instance.id}/?scenario_test_framework=true"
+
+    @property
+    def instance_kwargs(self):
+        return {'schematisation': self.schematisation.instance}
